@@ -32,14 +32,14 @@
 #' @family photon and energy ratio functions
 #'
 energy_ratio <- function(w.length, s.irrad,
-                         w.band.num=NULL, w.band.denom=NULL,
-                         unit.in="energy",
-                         check.spectrum=TRUE,
-                         use.cached.mult = getOption("photobiology.use.cached.mult", default = FALSE),
-                         use.hinges=getOption("photobiology.use.hinges", default=NULL) ) {
+                         w.band.num = NULL, w.band.denom = NULL,
+                         unit.in = "energy",
+                         check.spectrum = TRUE,
+                         use.cached.mult = FALSE,
+                         use.hinges = NULL) {
   return(waveband_ratio(w.length, s.irrad, w.band.num, w.band.denom,
-                        unit.out.num="energy", unit.out.denom="energy",
-                        unit.in=unit.in,
-                        check.spectrum=check.spectrum, use.cached.mult=use.cached.mult,
-                        use.hinges=use.hinges))
+                        unit.out.num = "energy", unit.out.denom = "energy",
+                        unit.in = unit.in,
+                        check.spectrum = check.spectrum, use.cached.mult = use.cached.mult,
+                        use.hinges = use.hinges))
 }
